-------------------------------------------------------
--! @file hgenerator.vhd
--! @brief Distance-3 Hamming Code Generator
--! @author Edson S. Gomi (gomi@usp.br)
--! @date 2019-04-09
-------------------------------------------------------

entity hgenerator is
  port (
    DIN :    in bit_vector  (3 downto 0);  -- Data in
    DOUT :   out bit_vector (6 downto 0)   -- Data out
    );
end entity hgenerator;

-- Code (7,3) 7 bits, 3 parity bits, 4 data bits
-- DOUT is 7 bit: i7 i6 i5 p4 i3 p2 p1
-- i7 i6 i5 and i3 are data bits
-- p4 p2 and p1 are parity bits

architecture hgenerator_arch of hgenerator is

  signal i7,i6,i5,i3 : bit;
  signal p4,p2,p1 :    bit;
  
begin

-- Signal Instantiation
 
  i7 <= DIN(3);
  i6 <= DIN(2);
  i5 <= DIN(1);
  i3 <= DIN(0);

  p4 <= (i7 xor i6) xor i5;
  p2 <= (i7 xor i6) xor i3;
  p1 <= (i7 xor i5) xor i3;  

  DOUT <= i7 & i6 & i5 & p4 & i3 & p2 & p1;
end architecture hgenerator_arch;
