-------------------------------------------------------
--! @file mux_tb.vhd
--! @brief Testbench for the multiplexer 4-to-1
--! @author Edson S. Gomi (gomi@usp.br)
--! @date 2020-04-28
-------------------------------------------------------

--  A testbench has no ports.

entity mux_tb is
end entity mux_tb;

architecture mux_tb_arch of mux_tb is

  --  Declaration of the component that will be instantiated.  
  component mux port (
    I : in bit_vector (3 downto 0);
    SEL : in bit_vector (1 downto 0);
    Y : out bit
    );
  end component;

  signal y : bit;
  signal sel : bit_vector (1 downto 0);
  signal i : bit_vector (3 downto 0);

begin
  --  Component instantiation.
  multiplexer: entity work.mux (with_select) port map (
    SEL => sel,
    I => i,
    Y => y
    );

  --  This process does the real job.
  stimulus_process: process is
    type pattern_type is record
      --  The inputs of the circuit.
      i : bit_vector (3 downto 0);
      sel : bit_vector (1 downto 0);
      
      --  The expected outputs of the circuit.
      y : bit;
    end record;

    --  The patterns to apply.
    type pattern_array is array (natural range <>) of pattern_type;
    constant patterns : pattern_array :=
      (("1010","00",'0'),
       ("1010","01",'1'),
       ("1010","10",'0'),
       ("1010","11",'1'));

  begin
    --  Initialize inputs.
    --  Nothing to do.
    
    wait for 5 ns;
    
    --  Check each pattern.
    for k in patterns'range loop
      --  Set the inputs.
      sel <= patterns(k).sel;
      i <= patterns(k).i;
  
  --  Wait for the results.
  wait for 3 ns;
  --  Check the outputs.
  assert y = patterns(k).y
    report "bad check" severity error;
end loop;
  assert false report "end of test" severity note;
  --  Wait forever; this will finish the simulation.
    wait;
    end process;

end mux_tb_arch;
  

